/*
 * JDBCRealm.java
 *
 * Created on September 5, 2007, 11:05 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.sun.test.realm;

import com.sun.appserv.security.AppservRealm;
import com.sun.enterprise.security.auth.realm.BadRealmException;
import com.sun.enterprise.security.auth.realm.InvalidOperationException;
import com.sun.enterprise.security.auth.realm.NoSuchRealmException;
import com.sun.enterprise.security.auth.realm.NoSuchUserException;
import com.sun.enterprise.security.auth.realm.User;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.Collections;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.Properties;
import org.postgresql.ds.PGSimpleDataSource;

/**
 *
 * @author hr124446
 */
public class JDBCRealm extends AppservRealm {
    
    private static final String AUTH_TYPE = "jdbcgrenade";
    
    private Connection connection;
    private PreparedStatement userStatement;
    private PreparedStatement groupStatement;
    private PreparedStatement authStatement;
    private String[] authenticationGroups;
    
    public JDBCRealm() {
        
        super();
    }
    
    public void init(Properties props)
    throws BadRealmException, NoSuchRealmException {
        
        super.init(props);
        
        PGSimpleDataSource dataSource;
        
        for (PropertyType propertyType : PropertyType.values()) {
            if (isEmpty(props, propertyType)) {
                throw new BadRealmException("Missing value for property(" +
                        propertyType.getPropertyKey() + ")");
            }
        }
        
        if (props.getProperty(AppservRealm.JAAS_CONTEXT_PARAM) != null) {
            setProperty(AppservRealm.JAAS_CONTEXT_PARAM,
                    props.getProperty(AppservRealm.JAAS_CONTEXT_PARAM));
        }
        
        dataSource = new PGSimpleDataSource();
        dataSource.setServerName(getProperty(props, PropertyType.DB_HOST));
        dataSource.setPortNumber(Integer.parseInt(getProperty(props,
                PropertyType.DB_PORT)));
        dataSource.setDatabaseName(getProperty(props, PropertyType.DB_INSTANCE));
        dataSource.setUser(getProperty(props, PropertyType.DB_USER));
        dataSource.setPassword(getProperty(props, PropertyType.DB_PASSWORD));
        
        authenticationGroups = new String[] {getProperty(props,
                PropertyType.AUTH_GROUP)};
        
        try {
            connection = dataSource.getConnection();
            connection.setAutoCommit(false);
            
            userStatement = connection.prepareStatement("SELECT \"" +
                    getProperty(props, PropertyType.COL_HANDLE) + "\" FROM \"" + getProperty(props,
                    PropertyType.TABLE_USER) + "\"");
            userStatement.setFetchSize(100);
            
            groupStatement = connection.prepareStatement("SELECT COUNT(*) FROM \""
                    + getProperty(props, PropertyType.TABLE_USER) + "\" WHERE \"" +
                    getProperty(props, PropertyType.COL_HANDLE) + "\"=?");
            groupStatement.setFetchSize(1);
            
            authStatement = connection.prepareStatement("SELECT COUNT(*) FROM \"" +
                    getProperty(props, PropertyType.TABLE_USER) + "\" WHERE \"" + getProperty(props,
                    PropertyType.COL_HANDLE) + "\"=? AND \""+ getProperty(props,
                    PropertyType.COL_PASSWORD) + "\"=? AND \"" + getProperty(props,
                    PropertyType.COL_VALID) + "\"='true' AND \"" + getProperty(props,
                    PropertyType.COL_ENABLED) + "\"='true'");
            authStatement.setFetchSize(1);
        } catch (SQLException sqlException) {
            throw new BadRealmException(sqlException);
        }
    }
    
    public String getProperty(Properties props, PropertyType propertyType) {
        
        return props.getProperty(propertyType.getPropertyKey());
    }
    
    public boolean isEmpty(Properties props, PropertyType propertyType) {
        
        return (getProperty(props, propertyType) == null) || (getProperty(props,
                propertyType).length() == 0);
    }
    
    public String getAuthType() {
        
        return AUTH_TYPE;
    }
    
    public synchronized Enumeration getUserNames()
    throws BadRealmException {
        
        ResultSet userNamesResultSet;
        LinkedList<String> userNameList;
        
        userNameList = new LinkedList<String>();
        
        try {
            userNamesResultSet = userStatement.executeQuery();
            
            while (userNamesResultSet.next()) {
                userNameList.add(userNamesResultSet.getString(1));
            }
            
            userNamesResultSet.close();
        } catch (SQLException sqlException) {
            throw new BadRealmException(sqlException);
        }
        
        return Collections.enumeration(userNameList);
    }
    
    public User getUser(String name)
    throws NoSuchUserException, BadRealmException {
        
        throw new BadRealmException(new UnsupportedOperationException());
    }
    
    public Enumeration getGroupNames()
    throws BadRealmException {
        
        return Collections.enumeration(Arrays.asList(authenticationGroups));
    }
    
    public synchronized Enumeration getGroupNames(String user)
    throws InvalidOperationException, NoSuchUserException {
        
        ResultSet groupResultSet;
        
        try {
            groupStatement.setString(1, user);
            groupResultSet = groupStatement.executeQuery();
            
            try {
                if (!groupResultSet.next()) {
                    throw new IllegalStateException("No count available for users on requested table");
                } else if (groupResultSet.getInt(1) == 0) {
                    throw new NoSuchUserException(user);
                } else {
                    return Collections.enumeration(Arrays.asList(authenticationGroups));
                }
            } finally {
                groupResultSet.close();
            }
        } catch (SQLException sqlException) {
            
            InvalidOperationException invalidOperationException;
            
            invalidOperationException = new
                    InvalidOperationException(sqlException.getMessage());
            invalidOperationException.initCause(sqlException);
            
            throw invalidOperationException;
        }
    }
    
    public void refresh() throws BadRealmException {
    }
    
    public synchronized String[] authenticateUser(String user, String password)
    throws SQLException {
        
        ResultSet authResultSet;
        
        authStatement.setString(1, user);
        authStatement.setString(2, password);
        authResultSet = authStatement.executeQuery();
        
        try {
            if (!authResultSet.next()) {
                throw new IllegalStateException("No count available for users on requested table");
            } else if (authResultSet.getInt(1) == 0) {
                return null;
            } else {
                return authenticationGroups;
            }
        } finally {
            authResultSet.close();
        }
    }
    
    protected void finalize()
    throws Throwable {
        
        userStatement.close();
        groupStatement.close();
        authStatement.close();
        connection.close();
        
        super.finalize();
    }
}

